import {Component, Directive, Provider, Input, Output, EventEmitter, HostBinding, HostListener, forwardRef} from 'angular2/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from 'angular2/common';

@Component({
    selector: 'sui-rating',
    template: `
<i class="icon"
   *ngFor="#icon of icons; #i = index"
   (mouseover)="mouseover(i)"
   (click)="click(i)"
   [class.selected]="_hoveredIndex >= i && !isReadonly"
   [class.active]="_value > i">
</i>
`,
    styles: [`:host.read-only .icon { cursor: auto }`]
})
export class Rating {
    @HostBinding('class.ui')
    @HostBinding('class.rating') ratingClasses = true;

    private _value:number = 0;
    private _max:number = 5;

    @Input()
    public set max(value:any) {
        this._max = parseInt(value);
    }
    private get icons() {
        return Array(this._max);
    }

    @Output() public valueChange:EventEmitter<number> = new EventEmitter(false);

    private _hoveredIndex:number = -1;
    private mouseover(i:number) {
        this._hoveredIndex = i;
    }
    @HostListener('mouseout')
    private mouseout() {
        this._hoveredIndex = -1;
    }

    private click(i:number) {
        if (!this.isReadonly) {
            this._value = i + 1;
            this.valueChange.emit(this._value);
        }
    }

    @HostBinding('class.read-only')
    @Input()
    public isReadonly:boolean = false;

    public writeValue(value:number) {
        this._value = value;
    }
}

const CUSTOM_VALUE_ACCESSOR = new Provider(NG_VALUE_ACCESSOR, {useExisting: forwardRef(() => RatingValueAccessor), multi: true});

@Directive({
    selector: 'sui-rating',
    host: {'(valueChange)': 'onChange($event)'},
    providers: [CUSTOM_VALUE_ACCESSOR]
})
export class RatingValueAccessor implements ControlValueAccessor {
    onChange = () => {};
    onTouched = () => {};

    constructor(private host: Rating) { }

    writeValue(value: any): void {
        this.host.writeValue(value);
    }

    registerOnChange(fn: () => void): void { this.onChange = fn; }
    registerOnTouched(fn: () => void): void { this.onTouched = fn; }
}